@@ -211,10 +211,18 @@ struct aws_s3_client_config {
     struct aws_signing_config_aws *signing_config;
 
     /* Size of parts the files will be downloaded or uploaded in. */
-    size_t part_size;
+    uint64_t part_size;
 
-    /* If the part size needs to be adjusted for service limits, this is the maximum size it will be adjusted to.. */
-    size_t max_part_size;
+    /* If the part size needs to be adjusted for service limits, this is the maximum size it will be adjusted to. On 32
+     * bit machine, it will be forced to SIZE_MAX, which is around 4GiB. The server limit is 5GiB, but object size limit
+     * is 5TiB for now. We should be good enough for all the cases. */
+    uint64_t max_part_size;
+
+    /* The size threshold in bytes for when to use multipart uploads for a AWS_S3_META_REQUEST_TYPE_PUT_OBJECT meta
+     * request. Uploads over this size will automatically use a multipart upload strategy,while uploads smaller or
+     * equal to this threshold will use a single request to upload the whole object. If not set, `part_size` will be
+     * used as threshold. */
+    uint64_t multipart_upload_threshold;
 
     /* Throughput target in Gbps that we are trying to reach. */
     double throughput_target_gbps;
@@ -266,6 +274,7 @@ struct aws_s3_client_config {
      * Optional.
      * Configuration options for connection monitoring.
      * If the transfer speed falls below the specified minimum_throughput_bytes_per_second, the operation is aborted.
+     * If set to NULL, default values are used.
      */
     struct aws_http_connection_monitoring_options *monitoring_options;
 
@@ -349,9 +358,18 @@ struct aws_s3_meta_request_options {
      * be used. If not NULL, these options will override the client options. */
     const struct aws_signing_config_aws *signing_config;
 
-    /* Initial HTTP message that defines what operation we are doing. */
+    /* Initial HTTP message that defines what operation we are doing.
+     * When uploading a file, you should set `send_filepath` (instead of the message's body-stream)
+     * for better performance. */
     struct aws_http_message *message;
 
+    /**
+     * Optional.
+     * If set, this file is sent as the request body, and the `message` body-stream is ignored.
+     * This can give better performance than sending data using the body-stream.
+     */
+    struct aws_byte_cursor send_filepath;
+
     /**
      * Optional.
      * if set, the flexible checksum will be performed by client based on the config.
@@ -553,7 +571,7 @@ int aws_s3_meta_request_pause(
  */
 struct aws_s3_upload_resume_token_options {
     struct aws_byte_cursor upload_id; /* Required */
-    size_t part_size;                 /* Required */
+    uint64_t part_size;               /* Required. Must be less than SIZE_MAX */
     size_t total_num_parts;           /* Required */
 
     /**
@@ -602,7 +620,7 @@ enum aws_s3_meta_request_type aws_s3_meta_request_resume_token_type(
  * Part size associated with operation.
  */
 AWS_S3_API
-size_t aws_s3_meta_request_resume_token_part_size(struct aws_s3_meta_request_resume_token *resume_token);
+uint64_t aws_s3_meta_request_resume_token_part_size(struct aws_s3_meta_request_resume_token *resume_token);
 
 /*
  * Total num parts associated with operation.
