@@ -58,7 +58,7 @@ static int s_test_s3_replace_quote_entities(struct aws_allocator *allocator, voi
         },
     };
 
-    for (size_t i = 0; i < (sizeof(test_cases) / sizeof(struct replace_quote_entities_test_case)); ++i) {
+    for (size_t i = 0; i < AWS_ARRAY_SIZE(test_cases); ++i) {
         struct replace_quote_entities_test_case *test_case = &test_cases[i];
 
         struct aws_byte_buf result_byte_buf;
@@ -116,7 +116,7 @@ static int s_test_s3_strip_quotes(struct aws_allocator *allocator, void *ctx) {
         },
     };
 
-    for (size_t i = 0; i < (sizeof(test_cases) / sizeof(struct strip_quotes_test_case)); ++i) {
+    for (size_t i = 0; i < AWS_ARRAY_SIZE(test_cases); ++i) {
         struct strip_quotes_test_case *test_case = &test_cases[i];
 
         struct aws_byte_buf result_byte_buf;
@@ -347,5 +347,69 @@ static int s_test_s3_get_num_parts_and_get_part_range(struct aws_allocator *allo
             s_validate_part_ranges(object_range_start, object_range_end, part_size, expected_num_parts, part_ranges));
     }
 
+    /* 1 byte range corner case. */
+    {
+        const uint32_t expected_num_parts = 1;
+        const uint64_t object_range_start = 8;
+        const uint64_t object_range_end = 8;
+
+        const uint64_t part_ranges[] = {8, 8};
+
+        ASSERT_TRUE(aws_s3_get_num_parts(part_size, object_range_start, object_range_end) == expected_num_parts);
+
+        ASSERT_SUCCESS(
+            s_validate_part_ranges(object_range_start, object_range_end, part_size, expected_num_parts, part_ranges));
+    }
+
     return 0;
 }
+
+AWS_TEST_CASE(test_s3_aws_xml_get_top_level_tag_with_root_name, s_test_s3_aws_xml_get_top_level_tag_with_root_name)
+static int s_test_s3_aws_xml_get_top_level_tag_with_root_name(struct aws_allocator *allocator, void *ctx) {
+    (void)allocator;
+    (void)ctx;
+
+    struct aws_byte_cursor example_error_body = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(
+        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+        "<Error>\n"
+        "<Code>AccessDenied</Code>\n"
+        "<Message>Access Denied</Message>\n"
+        "<RequestId>656c76696e6727732072657175657374</RequestId>\n"
+        "<HostId>Uuag1LuByRx9e6j5Onimru9pO4ZVKnJ2Qz7/C1NPcfTWAtRPfTaOFg==</HostId>\n"
+        "</Error>");
+
+    struct aws_byte_cursor example_success_body = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(
+        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+        "<CompleteMultipartUploadResult xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">\n"
+        "<Location>http://Example-Bucket.s3.<Region>.amazonaws.com/Example-Object</Location>\n"
+        "<Bucket>Example-Bucket</Bucket>\n"
+        "<Key>Example-Object</Key>\n"
+        "<ETag>\"3858f62230ac3c915f300c664312c11f-9\"</ETag>\n"
+        "</CompleteMultipartUploadResult>");
+
+    struct aws_byte_cursor error_body_xml_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Error");
+    struct aws_byte_cursor code_body_xml_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Code");
+    struct aws_byte_cursor etag_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ETag");
+
+    bool root_name_mismatch = false;
+    struct aws_string *error_code = aws_xml_get_top_level_tag_with_root_name(
+        allocator, &code_body_xml_name, &error_body_xml_name, &root_name_mismatch, &example_error_body);
+
+    ASSERT_NOT_NULL(error_code);
+    ASSERT_FALSE(root_name_mismatch);
+    ASSERT_TRUE(aws_string_eq_c_str(error_code, "AccessDenied"));
+    aws_string_destroy(error_code);
+
+    error_code = aws_xml_get_top_level_tag_with_root_name(
+        allocator, &code_body_xml_name, &error_body_xml_name, &root_name_mismatch, &example_success_body);
+    ASSERT_NULL(error_code);
+    ASSERT_TRUE(root_name_mismatch);
+
+    struct aws_string *etag = aws_xml_get_top_level_tag(allocator, &etag_header_name, &example_success_body);
+
+    ASSERT_NOT_NULL(etag);
+    ASSERT_TRUE(aws_string_eq_c_str(etag, "\"3858f62230ac3c915f300c664312c11f-9\""));
+    aws_string_destroy(etag);
+
+    return AWS_OP_SUCCESS;
+}
