@@ -13,17 +13,15 @@
  * permissions and limitations under the License.
  */
 
-#include <sys/param.h>
 #include <stdint.h>
-
-#include "error/s2n_errno.h"
-
-#include "utils/s2n_safety.h"
-#include "utils/s2n_mem.h"
+#include <sys/param.h>
 
 #include "crypto/s2n_hmac.h"
-
+#include "error/s2n_errno.h"
+#include "tls/s2n_connection.h"
 #include "tls/s2n_record.h"
+#include "utils/s2n_mem.h"
+#include "utils/s2n_safety.h"
 
 /* A TLS CBC record looks like ..
  *
@@ -47,12 +45,6 @@
  */
 int s2n_verify_cbc(struct s2n_connection *conn, struct s2n_hmac_state *hmac, struct s2n_blob *decrypted)
 {
-    /* Set up MAC copy workspace */
-    struct s2n_hmac_state *copy = &conn->client->record_mac_copy_workspace;
-    if (conn->mode == S2N_CLIENT) {
-       copy = &conn->server->record_mac_copy_workspace;
-    }
-    
     uint8_t mac_digest_size;
     POSIX_GUARD(s2n_hmac_digest_size(hmac->alg, &mac_digest_size));
 
@@ -69,7 +61,7 @@ int s2n_verify_cbc(struct s2n_connection *conn, struct s2n_hmac_state *hmac, str
 
     /* Update the MAC */
     POSIX_GUARD(s2n_hmac_update(hmac, decrypted->data, payload_length));
-    POSIX_GUARD(s2n_hmac_copy(copy, hmac));
+    int currently_in_hash_block = hmac->currently_in_hash_block;
 
     /* Check the MAC */
     uint8_t check_digest[S2N_MAX_DIGEST_LEN];
@@ -78,8 +70,12 @@ int s2n_verify_cbc(struct s2n_connection *conn, struct s2n_hmac_state *hmac, str
 
     int mismatches = s2n_constant_time_equals(decrypted->data + payload_length, check_digest, mac_digest_size) ^ 1;
 
-    /* Compute a MAC on the rest of the data so that we perform the same number of hash operations */
-    POSIX_GUARD(s2n_hmac_update(copy, decrypted->data + payload_length + mac_digest_size, decrypted->size - payload_length - mac_digest_size - 1));
+    /* Compute a MAC on the rest of the data so that we perform the same number of hash operations.
+     * Include the partial hash block from the first MAC to ensure we use the same number of blocks.
+     */
+    POSIX_GUARD(s2n_hmac_reset(hmac));
+    POSIX_GUARD(s2n_hmac_update(hmac, decrypted->data, currently_in_hash_block));
+    POSIX_GUARD(s2n_hmac_update(hmac, decrypted->data + payload_length + mac_digest_size, decrypted->size - payload_length - mac_digest_size - 1));
 
     /* SSLv3 doesn't specify what the padding should actually be */
     if (conn->actual_protocol_version == S2N_SSLv3) {
@@ -95,8 +91,6 @@ int s2n_verify_cbc(struct s2n_connection *conn, struct s2n_hmac_state *hmac, str
         mismatches |= (decrypted->data[j] ^ padding_length) & mask;
     }
 
-    POSIX_GUARD(s2n_hmac_reset(copy));
-
     S2N_ERROR_IF(mismatches, S2N_ERR_CBC_VERIFY);
 
     return 0;
