@@ -16,35 +16,34 @@
 #include <strings.h>
 #include <time.h>
 
-#include "error/s2n_errno.h"
-
 #include "crypto/s2n_certificate.h"
 #include "crypto/s2n_fips.h"
-
+#include "crypto/s2n_hkdf.h"
+#include "error/s2n_errno.h"
 #include "tls/s2n_cipher_preferences.h"
 #include "tls/s2n_internal.h"
+#include "tls/s2n_ktls.h"
 #include "tls/s2n_security_policies.h"
 #include "tls/s2n_tls13.h"
-#include "utils/s2n_safety.h"
-#include "crypto/s2n_hkdf.h"
-#include "utils/s2n_map.h"
 #include "utils/s2n_blob.h"
+#include "utils/s2n_map.h"
+#include "utils/s2n_safety.h"
 
 #if defined(CLOCK_MONOTONIC_RAW)
-#define S2N_CLOCK_HW CLOCK_MONOTONIC_RAW
+    #define S2N_CLOCK_HW CLOCK_MONOTONIC_RAW
 #else
-#define S2N_CLOCK_HW CLOCK_MONOTONIC
+    #define S2N_CLOCK_HW CLOCK_MONOTONIC
 #endif
 
 #define S2N_CLOCK_SYS CLOCK_REALTIME
 
 static int monotonic_clock(void *data, uint64_t *nanoseconds)
 {
-    struct timespec current_time = {0};
+    struct timespec current_time = { 0 };
 
     POSIX_GUARD(clock_gettime(S2N_CLOCK_HW, &current_time));
 
-    *nanoseconds = (uint64_t)current_time.tv_sec * 1000000000ull;
+    *nanoseconds = (uint64_t) current_time.tv_sec * 1000000000ull;
     *nanoseconds += current_time.tv_nsec;
 
     return 0;
@@ -52,19 +51,19 @@ static int monotonic_clock(void *data, uint64_t *nanoseconds)
 
 static int wall_clock(void *data, uint64_t *nanoseconds)
 {
-    struct timespec current_time = {0};
+    struct timespec current_time = { 0 };
 
     POSIX_GUARD(clock_gettime(S2N_CLOCK_SYS, &current_time));
 
-    *nanoseconds = (uint64_t)current_time.tv_sec * 1000000000ull;
+    *nanoseconds = (uint64_t) current_time.tv_sec * 1000000000ull;
     *nanoseconds += current_time.tv_nsec;
 
     return 0;
 }
 
-static struct s2n_config s2n_default_config = {0};
-static struct s2n_config s2n_default_fips_config = {0};
-static struct s2n_config s2n_default_tls13_config = {0};
+static struct s2n_config s2n_default_config = { 0 };
+static struct s2n_config s2n_default_fips_config = { 0 };
+static struct s2n_config s2n_default_tls13_config = { 0 };
 
 static int s2n_config_setup_default(struct s2n_config *config)
 {
@@ -135,8 +134,8 @@ static int s2n_config_cleanup(struct s2n_config *config)
 }
 
 static int s2n_config_update_domain_name_to_cert_map(struct s2n_config *config,
-                                                     struct s2n_blob *name,
-                                                     struct s2n_cert_chain_and_key *cert_key_pair)
+        struct s2n_blob *name,
+        struct s2n_cert_chain_and_key *cert_key_pair)
 {
     POSIX_ENSURE_REF(config);
     POSIX_ENSURE_REF(name);
@@ -151,7 +150,7 @@ static int s2n_config_update_domain_name_to_cert_map(struct s2n_config *config,
     bool key_found = false;
     POSIX_GUARD_RESULT(s2n_map_lookup(domain_name_to_cert_map, name, &s2n_map_value, &key_found));
     if (!key_found) {
-        struct certs_by_type value = {{ 0 }};
+        struct certs_by_type value = { { 0 } };
         value.certs[cert_type] = cert_key_pair;
         s2n_map_value.data = (uint8_t *) &value;
         s2n_map_value.size = sizeof(struct certs_by_type);
@@ -160,7 +159,7 @@ static int s2n_config_update_domain_name_to_cert_map(struct s2n_config *config,
         POSIX_GUARD_RESULT(s2n_map_add(domain_name_to_cert_map, name, &s2n_map_value));
         POSIX_GUARD_RESULT(s2n_map_complete(domain_name_to_cert_map));
     } else {
-        struct certs_by_type *value = (void *) s2n_map_value.data;;
+        struct certs_by_type *value = (void *) s2n_map_value.data;
         if (value->certs[cert_type] == NULL) {
             value->certs[cert_type] = cert_key_pair;
         } else if (config->cert_tiebreak_cb) {
@@ -185,7 +184,6 @@ static int s2n_config_update_domain_name_to_cert_map(struct s2n_config *config,
 
 static int s2n_config_build_domain_name_to_cert_map(struct s2n_config *config, struct s2n_cert_chain_and_key *cert_key_pair)
 {
-
     uint32_t cn_len = 0;
     POSIX_GUARD_RESULT(s2n_array_num_elements(cert_key_pair->cn_names, &cn_len));
     uint32_t san_len = 0;
@@ -194,13 +192,13 @@ static int s2n_config_build_domain_name_to_cert_map(struct s2n_config *config, s
     if (san_len == 0) {
         for (uint32_t i = 0; i < cn_len; i++) {
             struct s2n_blob *cn_name = NULL;
-            POSIX_GUARD_RESULT(s2n_array_get(cert_key_pair->cn_names, i, (void **)&cn_name));
+            POSIX_GUARD_RESULT(s2n_array_get(cert_key_pair->cn_names, i, (void **) &cn_name));
             POSIX_GUARD(s2n_config_update_domain_name_to_cert_map(config, cn_name, cert_key_pair));
         }
     } else {
         for (uint32_t i = 0; i < san_len; i++) {
             struct s2n_blob *san_name = NULL;
-            POSIX_GUARD_RESULT(s2n_array_get(cert_key_pair->san_names, i, (void **)&san_name));
+            POSIX_GUARD_RESULT(s2n_array_get(cert_key_pair->san_names, i, (void **) &san_name));
             POSIX_GUARD(s2n_config_update_domain_name_to_cert_map(config, san_name, cert_key_pair));
         }
     }
@@ -257,13 +255,13 @@ void s2n_wipe_static_configs(void)
 
 struct s2n_config *s2n_config_new(void)
 {
-    struct s2n_blob allocator = {0};
+    struct s2n_blob allocator = { 0 };
     struct s2n_config *new_config;
 
     PTR_GUARD_POSIX(s2n_alloc(&allocator, sizeof(struct s2n_config)));
     PTR_GUARD_POSIX(s2n_blob_zero(&allocator));
 
-    new_config = (struct s2n_config *)(void *)allocator.data;
+    new_config = (struct s2n_config *) (void *) allocator.data;
     if (s2n_config_init(new_config) != S2N_SUCCESS) {
         s2n_free(&allocator);
         return NULL;
@@ -289,11 +287,11 @@ static int s2n_verify_unique_ticket_key_comparator(const void *a, const void *b)
 int s2n_config_init_session_ticket_keys(struct s2n_config *config)
 {
     if (config->ticket_keys == NULL) {
-      POSIX_ENSURE_REF(config->ticket_keys = s2n_set_new(sizeof(struct s2n_ticket_key), s2n_config_store_ticket_key_comparator));
+        POSIX_ENSURE_REF(config->ticket_keys = s2n_set_new(sizeof(struct s2n_ticket_key), s2n_config_store_ticket_key_comparator));
     }
 
     if (config->ticket_key_hashes == NULL) {
-      POSIX_ENSURE_REF(config->ticket_key_hashes = s2n_set_new(SHA_DIGEST_LENGTH, s2n_verify_unique_ticket_key_comparator));
+        POSIX_ENSURE_REF(config->ticket_key_hashes = s2n_set_new(SHA_DIGEST_LENGTH, s2n_verify_unique_ticket_key_comparator));
     }
 
     return 0;
@@ -343,7 +341,7 @@ int s2n_config_free_dhparams(struct s2n_config *config)
         POSIX_GUARD(s2n_dh_params_free(config->dhparams));
     }
 
-    POSIX_GUARD(s2n_free_object((uint8_t **)&config->dhparams, sizeof(struct s2n_dh_params)));
+    POSIX_GUARD(s2n_free_object((uint8_t **) &config->dhparams, sizeof(struct s2n_dh_params)));
     return 0;
 }
 
@@ -359,7 +357,7 @@ int s2n_config_free(struct s2n_config *config)
 {
     s2n_config_cleanup(config);
 
-    POSIX_GUARD(s2n_free_object((uint8_t **)&config, sizeof(struct s2n_config)));
+    POSIX_GUARD(s2n_free_object((uint8_t **) &config, sizeof(struct s2n_config)));
     return 0;
 }
 
@@ -436,7 +434,6 @@ int s2n_config_set_max_cert_chain_depth(struct s2n_config *config, uint16_t max_
     return 0;
 }
 
-
 int s2n_config_set_status_request_type(struct s2n_config *config, s2n_status_request_type type)
 {
     S2N_ERROR_IF(type == S2N_STATUS_REQUEST_OCSP && !s2n_x509_ocsp_stapling_supported(), S2N_ERR_OCSP_NOT_SUPPORTED);
@@ -563,8 +560,8 @@ static int s2n_config_clear_default_certificates(struct s2n_config *config)
 }
 
 int s2n_config_set_cert_chain_and_key_defaults(struct s2n_config *config,
-                                               struct s2n_cert_chain_and_key **cert_key_pairs,
-                                               uint32_t num_cert_key_pairs)
+        struct s2n_cert_chain_and_key **cert_key_pairs,
+        uint32_t num_cert_key_pairs)
 {
     POSIX_ENSURE_REF(config);
     POSIX_ENSURE_REF(cert_key_pairs);
@@ -577,7 +574,7 @@ int s2n_config_set_cert_chain_and_key_defaults(struct s2n_config *config,
     POSIX_ENSURE(config->cert_ownership != S2N_LIB_OWNED, S2N_ERR_CERT_OWNERSHIP);
 
     /* Validate certs being set before clearing auto-chosen defaults or previously set defaults */
-    struct certs_by_type new_defaults = {{ 0 }};
+    struct certs_by_type new_defaults = { { 0 } };
     for (uint32_t i = 0; i < num_cert_key_pairs; i++) {
         POSIX_ENSURE_REF(cert_key_pairs[i]);
         s2n_pkey_type cert_type = s2n_cert_chain_and_key_get_pkey_type(cert_key_pairs[i]);
@@ -599,14 +596,14 @@ int s2n_config_set_cert_chain_and_key_defaults(struct s2n_config *config,
 
 int s2n_config_add_dhparams(struct s2n_config *config, const char *dhparams_pem)
 {
-    DEFER_CLEANUP(struct s2n_stuffer dhparams_in_stuffer = {0}, s2n_stuffer_free);
-    DEFER_CLEANUP(struct s2n_stuffer dhparams_out_stuffer = {0}, s2n_stuffer_free);
-    struct s2n_blob dhparams_blob = {0};
-    struct s2n_blob mem = {0};
+    DEFER_CLEANUP(struct s2n_stuffer dhparams_in_stuffer = { 0 }, s2n_stuffer_free);
+    DEFER_CLEANUP(struct s2n_stuffer dhparams_out_stuffer = { 0 }, s2n_stuffer_free);
+    struct s2n_blob dhparams_blob = { 0 };
+    struct s2n_blob mem = { 0 };
 
     /* Allocate the memory for the chain and key struct */
     POSIX_GUARD(s2n_alloc(&mem, sizeof(struct s2n_dh_params)));
-    config->dhparams = (struct s2n_dh_params *)(void *)mem.data;
+    config->dhparams = (struct s2n_dh_params *) (void *) mem.data;
 
     if (s2n_stuffer_alloc_ro_from_string(&dhparams_in_stuffer, dhparams_pem) != S2N_SUCCESS) {
         s2n_free(&mem);
@@ -629,7 +626,7 @@ int s2n_config_add_dhparams(struct s2n_config *config, const char *dhparams_pem)
     return 0;
 }
 
-extern int s2n_config_set_wall_clock(struct s2n_config *config, s2n_clock_time_nanoseconds clock_fn, void *ctx)
+int s2n_config_set_wall_clock(struct s2n_config *config, s2n_clock_time_nanoseconds clock_fn, void *ctx)
 {
     POSIX_ENSURE_REF(clock_fn);
 
@@ -639,7 +636,7 @@ extern int s2n_config_set_wall_clock(struct s2n_config *config, s2n_clock_time_n
     return 0;
 }
 
-extern int s2n_config_set_monotonic_clock(struct s2n_config *config, s2n_clock_time_nanoseconds clock_fn, void *ctx)
+int s2n_config_set_monotonic_clock(struct s2n_config *config, s2n_clock_time_nanoseconds clock_fn, void *ctx)
 {
     POSIX_ENSURE_REF(clock_fn);
 
@@ -716,8 +713,7 @@ int s2n_config_set_client_hello_cb(struct s2n_config *config, s2n_client_hello_f
 int s2n_config_set_client_hello_cb_mode(struct s2n_config *config, s2n_client_hello_cb_mode cb_mode)
 {
     POSIX_ENSURE_REF(config);
-    POSIX_ENSURE(cb_mode == S2N_CLIENT_HELLO_CB_BLOCKING ||
-            cb_mode == S2N_CLIENT_HELLO_CB_NONBLOCKING, S2N_ERR_INVALID_STATE);
+    POSIX_ENSURE(cb_mode == S2N_CLIENT_HELLO_CB_BLOCKING || cb_mode == S2N_CLIENT_HELLO_CB_NONBLOCKING, S2N_ERR_INVALID_STATE);
 
     config->client_hello_cb_mode = cb_mode;
     return S2N_SUCCESS;
@@ -744,7 +740,7 @@ int s2n_config_accept_max_fragment_length(struct s2n_config *config)
 }
 
 int s2n_config_set_session_state_lifetime(struct s2n_config *config,
-                                          uint64_t lifetime_in_secs)
+        uint64_t lifetime_in_secs)
 {
     POSIX_ENSURE_REF(config);
 
@@ -785,8 +781,7 @@ int s2n_config_set_session_cache_onoff(struct s2n_config *config, uint8_t enable
     if (enabled && config->cache_store && config->cache_retrieve && config->cache_delete) {
         POSIX_GUARD(s2n_config_init_session_ticket_keys(config));
         config->use_session_cache = 1;
-    }
-    else {
+    } else {
         if (!config->use_tickets) {
             POSIX_GUARD(s2n_config_free_session_ticket_keys(config));
         }
@@ -796,7 +791,7 @@ int s2n_config_set_session_cache_onoff(struct s2n_config *config, uint8_t enable
 }
 
 int s2n_config_set_ticket_encrypt_decrypt_key_lifetime(struct s2n_config *config,
-                                                       uint64_t lifetime_in_secs)
+        uint64_t lifetime_in_secs)
 {
     POSIX_ENSURE_REF(config);
 
@@ -805,7 +800,7 @@ int s2n_config_set_ticket_encrypt_decrypt_key_lifetime(struct s2n_config *config
 }
 
 int s2n_config_set_ticket_decrypt_key_lifetime(struct s2n_config *config,
-                                               uint64_t lifetime_in_secs)
+        uint64_t lifetime_in_secs)
 {
     POSIX_ENSURE_REF(config);
 
@@ -814,9 +809,9 @@ int s2n_config_set_ticket_decrypt_key_lifetime(struct s2n_config *config,
 }
 
 int s2n_config_add_ticket_crypto_key(struct s2n_config *config,
-                                     const uint8_t *name, uint32_t name_len,
-                                     uint8_t *key, uint32_t key_len,
-                                     uint64_t intro_time_in_seconds_from_epoch)
+        const uint8_t *name, uint32_t name_len,
+        uint8_t *key, uint32_t key_len,
+        uint64_t intro_time_in_seconds_from_epoch)
 {
     POSIX_ENSURE_REF(config);
     POSIX_ENSURE_REF(name);
@@ -847,10 +842,14 @@ int s2n_config_add_ticket_crypto_key(struct s2n_config *config,
     POSIX_ENSURE(s2n_find_ticket_key(config, name_data) == NULL, S2N_ERR_INVALID_TICKET_KEY_NAME_OR_NAME_LENGTH);
 
     uint8_t output_pad[S2N_AES256_KEY_LEN + S2N_TICKET_AAD_IMPLICIT_LEN] = { 0 };
-    struct s2n_blob out_key = { .data = output_pad, .size = s2n_array_len(output_pad) };
-    struct s2n_blob in_key = { .data = key, .size = key_len };
-    struct s2n_blob salt = { .size = 0 };
-    struct s2n_blob info = { .size = 0 };
+    struct s2n_blob out_key = { 0 };
+    POSIX_GUARD(s2n_blob_init(&out_key, output_pad, s2n_array_len(output_pad)));
+    struct s2n_blob in_key = { 0 };
+    POSIX_GUARD(s2n_blob_init(&in_key, key, key_len));
+    struct s2n_blob salt = { 0 };
+    POSIX_GUARD(s2n_blob_init(&salt, NULL, 0));
+    struct s2n_blob info = { 0 };
+    POSIX_GUARD(s2n_blob_init(&info, NULL, 0));
 
     struct s2n_ticket_key *session_ticket_key = { 0 };
     DEFER_CLEANUP(struct s2n_blob allocator = { 0 }, s2n_free);
@@ -944,7 +943,8 @@ int s2n_config_set_psk_selection_callback(struct s2n_config *config, s2n_psk_sel
     return S2N_SUCCESS;
 }
 
-int s2n_config_set_key_log_cb(struct s2n_config *config, s2n_key_log_fn callback, void *ctx) {
+int s2n_config_set_key_log_cb(struct s2n_config *config, s2n_key_log_fn callback, void *ctx)
+{
     POSIX_ENSURE_MUT(config);
 
     config->key_log_cb = callback;
@@ -953,10 +953,11 @@ int s2n_config_set_key_log_cb(struct s2n_config *config, s2n_key_log_fn callback
     return S2N_SUCCESS;
 }
 
-int s2n_config_set_async_pkey_validation_mode(struct s2n_config *config, s2n_async_pkey_validation_mode mode) {
+int s2n_config_set_async_pkey_validation_mode(struct s2n_config *config, s2n_async_pkey_validation_mode mode)
+{
     POSIX_ENSURE_REF(config);
 
-    switch(mode) {
+    switch (mode) {
         case S2N_ASYNC_PKEY_VALIDATION_FAST:
         case S2N_ASYNC_PKEY_VALIDATION_STRICT:
             config->async_pkey_validation_mode = mode;
@@ -966,7 +967,8 @@ int s2n_config_set_async_pkey_validation_mode(struct s2n_config *config, s2n_asy
     POSIX_BAIL(S2N_ERR_INVALID_ARGUMENT);
 }
 
-int s2n_config_set_ctx(struct s2n_config *config, void *ctx) {
+int s2n_config_set_ctx(struct s2n_config *config, void *ctx)
+{
     POSIX_ENSURE_REF(config);
 
     config->context = ctx;
@@ -974,7 +976,8 @@ int s2n_config_set_ctx(struct s2n_config *config, void *ctx) {
     return S2N_SUCCESS;
 }
 
-int s2n_config_get_ctx(struct s2n_config *config, void **ctx) {
+int s2n_config_get_ctx(struct s2n_config *config, void **ctx)
+{
     POSIX_ENSURE_REF(config);
     POSIX_ENSURE_REF(ctx);
 
@@ -988,7 +991,8 @@ int s2n_config_get_ctx(struct s2n_config *config, void **ctx) {
  *
  * Polling means that the callback function can be called multiple times.
  */
-int s2n_config_client_hello_cb_enable_poll(struct s2n_config *config) {
+int s2n_config_client_hello_cb_enable_poll(struct s2n_config *config)
+{
     POSIX_ENSURE_REF(config);
 
     config->client_hello_cb_enable_poll = 1;
@@ -996,9 +1000,10 @@ int s2n_config_client_hello_cb_enable_poll(struct s2n_config *config) {
     return S2N_SUCCESS;
 }
 
-int s2n_config_set_send_buffer_size(struct s2n_config *config, uint32_t size) {
+int s2n_config_set_send_buffer_size(struct s2n_config *config, uint32_t size)
+{
     POSIX_ENSURE_REF(config);
-    POSIX_ENSURE(size > S2N_TLS_MAX_RECORD_LEN_FOR(0), S2N_ERR_INVALID_ARGUMENT);
+    POSIX_ENSURE(size >= S2N_MIN_SEND_BUFFER_SIZE, S2N_ERR_INVALID_ARGUMENT);
     config->send_buffer_size_override = size;
     return S2N_SUCCESS;
 }
@@ -1050,3 +1055,20 @@ S2N_RESULT s2n_config_wall_clock(struct s2n_config *config, uint64_t *output)
     RESULT_ENSURE(config->wall_clock(config->sys_clock_ctx, output) >= S2N_SUCCESS, S2N_ERR_CANCELLED);
     return S2N_RESULT_OK;
 }
+
+int s2n_config_set_crl_lookup_cb(struct s2n_config *config, s2n_crl_lookup_callback cb, void *ctx)
+{
+    POSIX_ENSURE_REF(config);
+    config->crl_lookup_cb = cb;
+    config->crl_lookup_ctx = ctx;
+    return S2N_SUCCESS;
+}
+
+int s2n_config_set_recv_multi_record(struct s2n_config *config, bool enabled)
+{
+    POSIX_ENSURE_REF(config);
+
+    config->recv_multi_record = enabled;
+
+    return S2N_SUCCESS;
+}
