@@ -72,6 +72,7 @@ S2N_RESULT s2n_record_max_write_payload_size(struct s2n_connection *conn, uint16
     RESULT_ENSURE_REF(conn);
     RESULT_ENSURE_REF(conn->config);
     RESULT_ENSURE_MUT(max_fragment_size);
+    RESULT_ENSURE(conn->max_outgoing_fragment_length > 0, S2N_ERR_FRAGMENT_LENGTH_TOO_SMALL);
 
     *max_fragment_size = MIN(conn->max_outgoing_fragment_length, S2N_TLS_MAXIMUM_FRAGMENT_LENGTH);
 
@@ -89,9 +90,6 @@ S2N_RESULT s2n_record_max_write_payload_size(struct s2n_connection *conn, uint16
         }
     }
 
-    /* Ensure that we don't reserve too little space.  */
-    RESULT_ENSURE(*max_fragment_size >= S2N_MAX_FRAGMENT_LENGTH_MIN, S2N_ERR_FRAGMENT_LENGTH_TOO_SMALL);
-
     return S2N_RESULT_OK;
 }
 
@@ -159,20 +157,42 @@ S2N_RESULT s2n_record_min_write_payload_size(struct s2n_connection *conn, uint16
 int s2n_record_write_protocol_version(struct s2n_connection *conn, struct s2n_stuffer *out)
 {
     uint8_t record_protocol_version = conn->actual_protocol_version;
+
+    /**
+     *= https://www.rfc-editor.org/rfc/rfc8446#section-5.1
+     *# This version value is historical, deriving from the use of 0x0301 for
+     *# TLS 1.0 and 0x0300 for SSL 3.0.  In order to maximize backward
+     *# compatibility, a record containing an initial ClientHello SHOULD have
+     *# version 0x0301 (reflecting TLS 1.0)
+     *
+     * We set actual_protocol_version early for clients, but we do not
+     * use that assumed value here in case we are talking to a legacy
+     * server that expects TLS1.0.
+     *
+     * If we are requesting early data, we can assume that we aren't talking to
+     * a legacy server as a legacy server would not know how to handle early data.
+     **/
     if (conn->server_protocol_version == s2n_unknown_protocol_version
             && conn->early_data_state != S2N_EARLY_DATA_REQUESTED) {
-        /* Some legacy TLS implementations can't handle records with protocol version higher than TLS1.0.
-         * To provide maximum compatibility, send record version as TLS1.0 if server protocol version isn't
-         * established yet, which happens only during ClientHello message. Note, this has no effect on
-         * protocol version in ClientHello, so we're still able to negotiate protocol versions above TLS1.0 */
         record_protocol_version = MIN(record_protocol_version, S2N_TLS10);
     }
 
-    /* In accordance to TLS 1.3 spec, https://tools.ietf.org/html/rfc8446#section-5.1
-     * tls record version should never be greater than 33 (legacy TLS 1.2 version).
-     */
+    /**
+     *= https://www.rfc-editor.org/rfc/rfc8446#section-5.1
+     *# legacy_record_version:  MUST be set to 0x0303 for all records
+     *#    generated by a TLS 1.3 implementation other than an initial
+     *#    ClientHello (i.e., one not generated after a HelloRetryRequest),
+     *#    where it MAY also be 0x0301 for compatibility purposes.
+     **/
     record_protocol_version = MIN(record_protocol_version, S2N_TLS12);
 
+    /* Never send an empty protocol version.
+     * If the protocol version is unknown, default to TLS1.0 like we do for initial ClientHellos.
+     */
+    if (record_protocol_version == s2n_unknown_protocol_version) {
+        record_protocol_version = S2N_TLS10;
+    }
+
     uint8_t protocol_version[S2N_TLS_PROTOCOL_VERSION_LEN];
     protocol_version[0] = record_protocol_version / 10;
     protocol_version[1] = record_protocol_version % 10;
@@ -423,7 +443,7 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
                  * NOTE: We can't use the same random IV blob as both the initial block and IV since it will result in:
                  * AES(Key, XOR(random_iv, random_iv)) == AES(Key, 0), which will be shared by all records in this session.
                  */
-                struct s2n_blob explicit_iv_placeholder;
+                struct s2n_blob explicit_iv_placeholder = { 0 };
                 uint8_t zero_block[S2N_TLS_MAX_IV_LEN] = { 0 };
                 POSIX_GUARD(s2n_blob_init(&explicit_iv_placeholder, zero_block, block_size));
                 POSIX_GUARD_RESULT(s2n_get_public_random_data(&explicit_iv_placeholder));
@@ -438,7 +458,8 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     }
 
     /* We are done with this sequence number, so we can increment it */
-    struct s2n_blob seq = { .data = sequence_number, .size = S2N_TLS_SEQUENCE_NUM_LEN };
+    struct s2n_blob seq = { 0 };
+    POSIX_GUARD(s2n_blob_init(&seq, sequence_number, S2N_TLS_SEQUENCE_NUM_LEN));
     POSIX_GUARD(s2n_increment_sequence_number(&seq));
 
     /* Write the plaintext data */
@@ -519,10 +540,13 @@ int s2n_record_writev(struct s2n_connection *conn, uint8_t content_type, const s
     return data_bytes_to_take;
 }
 
-int s2n_record_write(struct s2n_connection *conn, uint8_t content_type, struct s2n_blob *in)
+S2N_RESULT s2n_record_write(struct s2n_connection *conn, uint8_t content_type, struct s2n_blob *in)
 {
     struct iovec iov;
     iov.iov_base = in->data;
     iov.iov_len = in->size;
-    return s2n_record_writev(conn, content_type, &iov, 1, 0, in->size);
+    int written = s2n_record_writev(conn, content_type, &iov, 1, 0, in->size);
+    RESULT_GUARD_POSIX(written);
+    RESULT_ENSURE(written == in->size, S2N_ERR_FRAGMENT_LENGTH_TOO_LARGE);
+    return S2N_RESULT_OK;
 }
