@@ -76,8 +76,9 @@ int s2n_server_nst_send(struct s2n_connection *conn)
 {
     uint16_t session_ticket_len = S2N_TLS12_TICKET_SIZE_IN_BYTES;
     uint8_t data[S2N_TLS12_TICKET_SIZE_IN_BYTES] = { 0 };
-    struct s2n_blob entry = { .data = data, .size = sizeof(data) };
-    struct s2n_stuffer to;
+    struct s2n_blob entry = { 0 };
+    POSIX_GUARD(s2n_blob_init(&entry, data, sizeof(data)));
+    struct s2n_stuffer to = { 0 };
     uint32_t lifetime_hint_in_secs =
             (conn->config->encrypt_decrypt_key_lifetime_in_nanos + conn->config->decrypt_key_lifetime_in_nanos) / ONE_SEC_IN_NANOS;
 
@@ -149,21 +150,6 @@ S2N_RESULT s2n_tls13_server_nst_send(struct s2n_connection *conn, s2n_blocked_st
 
     RESULT_ENSURE(conn->tickets_sent <= conn->tickets_to_send, S2N_ERR_INTEGER_OVERFLOW);
 
-    /* Flush any buffered records to ensure an empty output buffer.
-     *
-     * This is important when buffering multiple records because we don't:
-     * 1) Respect max fragment length for handshake messages
-     * 2) Check if there is sufficient space in the output buffer for
-     *    post-handshake messages.
-     *
-     * Careful how this flush fits into the ordering of this method:
-     * We want to avoid unnecessary work like repeatedly encrypting
-     * the ticket if the flush repeatedly blocks, but we also want
-     * to avoid unnecessarily flushing if no ticket will actually
-     * need to be sent.
-     */
-    RESULT_GUARD_POSIX(s2n_flush(conn, blocked));
-
     size_t session_state_size = 0;
     RESULT_GUARD(s2n_connection_get_session_state_size(conn, &session_state_size));
     const size_t maximum_nst_size = session_state_size + S2N_TLS13_MAX_FIXED_NEW_SESSION_TICKET_SIZE;
@@ -176,18 +162,9 @@ S2N_RESULT s2n_tls13_server_nst_send(struct s2n_connection *conn, s2n_blocked_st
             return S2N_RESULT_OK;
         }
 
-        struct s2n_blob nst_blob = { 0 };
-        uint16_t nst_size = s2n_stuffer_data_available(nst_stuffer);
-        uint8_t *nst_data = s2n_stuffer_raw_read(nst_stuffer, nst_size);
-        RESULT_ENSURE_REF(nst_data);
-        RESULT_GUARD_POSIX(s2n_blob_init(&nst_blob, nst_data, nst_size));
-
-        RESULT_GUARD_POSIX(s2n_record_write(conn, TLS_HANDSHAKE, &nst_blob));
-        RESULT_GUARD_POSIX(s2n_flush(conn, blocked));
-        RESULT_GUARD_POSIX(s2n_stuffer_wipe(nst_stuffer));
+        RESULT_GUARD(s2n_post_handshake_write_records(conn, blocked));
     }
 
-    RESULT_GUARD_POSIX(s2n_stuffer_resize(nst_stuffer, 0));
     return S2N_RESULT_OK;
 }
 
